#!/usr/bin/python
import sys
import os
import re

sub_counter = 0

add_pattern = r'''(?s)\{\+\+(?P<value>.*?)\+\+[ \t]*(\[(?P<meta>.*?)\])?[ \t]*\}'''

del_pattern = r'''(?s)\{\-\-(?P<value>.*?)\-\-[ \t]*(\[(?P<meta>.*?)\])?[ \t]*\}'''

insdel_comm_pattern = r'''(?s)(?<=[\+\-=~<]\})[ \t]*\{\>\>(?P<value>.*?)\<\<\}'''
comm_pattern = r'''(?s)\{\>\>(?P<value>.*?)\<\<\}'''

gen_comm_pattern = r'''(?s)\{[ \t]*\[(?P<meta>.*?)\][ \t]*\}'''

subs_pattern = r'''(?s)\{\~\~(?P<original>(?:[^\~\>]|(?:\~(?!\>)))+)\~\>(?P<new>(?:[^\~\~]|(?:\~(?!\~\})))+)\~\~\}'''

mark_pattern = r'''(?s)\{\=\=(?P<value>.*?)\=\=\}'''

fix_grafs_pattern = r'''^(?P<value><(del|ins|span|mark).*?)$'''

test_pattern = '''{~~Eighty-seven~>Four score and seven~~} years ago our fathers brought forth on this continent a new {~~state~>nation~~}, conceived in liberty, and dedicated to the proposition that all men {--and women--}{>>Tackle this after the war<<} are created equal.'''



def deletionProcess(group_object):
	replaceString = ''
	if group_object.group('value') == '\n\n':
		replaceString = "<del>&nbsp;</del>"
	else:
		replaceString = '<del class="crit">' + group_object.group('value').replace("\n\n", "</del>\n\n<del class=\"crit\">") + '</del>'
	return replaceString



def subsProcess(group_object):
	global sub_counter
	sub_counter += 1
	delString = '<del class="crit" data-subout="sub' + str(sub_counter) + '">' + group_object.group('original') + '</del>'
	insString  = '<ins class="crit" id="sub' + str(sub_counter) + '">' + group_object.group('new') + '</ins>'
	return delString + insString


# Converts Addition markup to HTML
def additionProcess(group_object):
	replaceString = ''

	# Is there a new paragraph followed by new text
	if group_object.group('value').startswith('\n\n') and group_object.group('value') != "\n\n":
		replaceString = "\n\n<span style=\"display:none\"></span><ins class=\"crit criticbreak\">&nbsp;</ins>\n\n"
		replaceString = replaceString + '<span style=\"display:none\"></span><ins class="crit">' + group_object.group('value').replace("\n\n", "</ins>\n\n<ins class=\"crit\">")
		replaceString = replaceString +  '</ins>'


	# Is the addition just a single new paragraph
	elif group_object.group('value') == "\n\n":
		replaceString = "\n\n<span style=\"display:none\"></span><ins class=\"crit criticbreak\">&nbsp;" + "</ins>\n\n"

	# Is it added text followed by a new paragraph?
	elif group_object.group('value').endswith('\n\n') and group_object.group('value') != "\n\n":
		replaceString = '<span style=\"display:none\"></span><ins class="crit">' + group_object.group('value').replace("\n\n", "</ins>\n\n<span style=\"display:none\"></span><ins class=\"crit\"><br>") + '</ins>'
		replaceString = replaceString + "\n\n<span style=\"display:none\"></span><ins class=\"crit criticbreak\">&nbsp;</ins>\n\n"

	else:
		replaceString = '<span style=\"display:none\"></span><ins class="crit">' + group_object.group('value').replace("\n\n", "</ins>\n\n<span style=\"display:none\"></span><ins class=\"crit\">") + '</ins>'

	return replaceString

def insDelHighlightProcess(group_object):
	content = group_object.group('value').replace("\n", " ")
	replaceString = '<span class="critic criticcomment inline" data-comment="' + content + '">&dagger;</span>'
	return replaceString

def highlightProcess(group_object):
	content = group_object.group('value').replace("\n", " ")
	replaceString = '<span class="critic criticcomment">' + content + '</span>'
	return replaceString


def markProcess(group_object):
	replaceString = '<mark class="crit">' + group_object.group('value') + '</mark>'
	return replaceString

def fixGrafsProcess(group_object):
	replaceString = '<p>' + group_object.group('value') + '</p>'
	return replaceString

# a = '''
# <div id="criticnav"><ul><li id="markup-button">Markup</li><li id="original-button">Original</li><li id="edited-button">Edited</li></ul></div>
# '''


# Accept input from Marked.app

h = sys.stdin.read()

#h = test_pattern


h = re.sub(insdel_comm_pattern, insDelHighlightProcess, h, flags=re.DOTALL)

h = re.sub(del_pattern, deletionProcess, h, flags=re.DOTALL)

h = re.sub(add_pattern, additionProcess, h, flags=re.DOTALL)

h = re.sub(comm_pattern, highlightProcess, h, flags=re.DOTALL)

h = re.sub(mark_pattern, markProcess, h, flags=re.DOTALL)

h = re.sub(subs_pattern, subsProcess, h, flags=re.DOTALL)

h = re.sub(fix_grafs_pattern, fixGrafsProcess, h, flags=re.DOTALL | re.MULTILINE)

# print h
# z = h + a
# sys.stdout.write(z)
sys.stdout.write(h)
