#!/System/Library/Frameworks/Ruby.framework/Versions/Current/usr/bin/ruby
# encoding: utf-8
require 'shellwords'
require 'fileutils'
require 'cgi'
require 'rexml/document'

def class_exists?(class_name)
    klass = Module.const_get(class_name)
    return klass.is_a?(Class)
    rescue NameError
    return false
end

if class_exists? 'Encoding'
    Encoding.default_external = Encoding::UTF_8 if Encoding.respond_to?('default_external')
    Encoding.default_internal = Encoding::UTF_8 if Encoding.respond_to?('default_internal')
end

class String

  def hexlify
    self.split("").collect { |c|
      c[0].to_s(16)
    }.join
  end

  def unhexlify
    self.scan(/../).collect { |c|
      c.to_i(16).chr
    }.join
  end

  def safename
    self.gsub(/[^\w\.\-]/,"_")
  end

  def extract_images(folder, relative_root=nil)
    input = self.dup
    embed_re = /\\*\{\\\*\\shppict \{\\pict.*?filename (.*?)\}.*?picw(\d+).*?pich(\d+).*?\\(\w+)blip (.*?)\}\}(\\\n)?/m
    input.gsub!(embed_re) do |m|
      name = $1
      ext = $4
      hex = $5
      newline = $6 ? "\\\n\\\n" : "\\\n"
      # width = $2
      # height = $3

      base_folder = File.expand_path(folder)
      target = File.join(base_folder, "#{name}.#{ext}".safename)

      FileUtils.mkdir_p(base_folder)

      File.open(target, 'w+') do |f|
        f.binmode
        f.puts hex.unhexlify
      end

      if relative_root
        target = File.expand_path(target).sub(/^#{relative_root}\//,'')
      end

      "![#{name}](#{target.gsub(/ /,'%20')})#{newline}"
    end
    input
  end

  def convert_linked_images(relative_root=nil)
    input = self.dup
    image_re = /\\*\{\$SCRImageLink\[w:(\d+);h:(\d+)\]=(.*?)\\\}(\\\n)?/
    input.gsub!(image_re) do |m|
      path = $3.strip
      name = File.basename(path).sub(/\.\w{3,4}/,'')
      newline = $4 ? "\\\n\\\n" : "\\\n"
      # width = $1
      # height = $2

      if relative_root
        path = File.expand_path(path).sub(/^#{relative_root}\//,'')
      end

      "![#{name}](#{path.gsub(/ /,'%20')})#{newline}"
    end
    input
  end

  def markdownify
    input = self.dup
    input.sub!(/(?:.|\n)*?<body>((?:.|\n)*)<\/body>(?:.|\n)*/, '\1')
    input.gsub!(/<\/?p.*?>/, '')
    input.gsub!(/(<br>\n?)+/, "\n")
    input.gsub!(/<span.*?>(.*?)<\/span>/, '\1')
    input.gsub!(/&gt;/, '>')
    input.gsub!(/&lt;/, '<')
    input.gsub!(/((?=\S)<b>|<\/b>(?<=\S))/, '**')
    input.gsub!(/((?=\S)<i>|<\/i>(?<=\S))/, '_')
    input.gsub!(/<a.*?href="(.*?)".*?>(.*?)<\/a>/, '[\2](\1)')
    input + "\n"
  end
end


# scrivcompile.rb "path/to/doc.scriv" useheaders(true/false)

def get_children(ele,path,files,depth)
  ele.elements.each('*/BinderItem') do |child|
        # Ignore docs set to not include in compile
        includetag = REXML::XPath.first( child, "MetaData/IncludeInCompile" )
        if !includetag.nil? && includetag.text == "Yes"
          id = child.attributes["ID"]
          # passing type, would eventually use to control header/title output
          type = child.attributes["Type"]
          elArray = child.elements.to_a
          if (elArray.length > 0)
            title = elArray[0].text
            title = title.force_encoding('utf-8') if title.respond_to?('force_encoding')
          else
            title = ''
          end
          file = "#{path}/Files/Docs/#{id}.rtf"
          filepath = File.exists?(file) ? file : false
          files << { 'path' => filepath, 'title' => title, 'depth' => depth, 'type' => type }
        end
        get_children(child,path,files,depth+1)
  end
end

def expand_scriv_path(scrivpath)
  scrivpath = File.expand_path(scrivpath)
  scrivx = File.basename("#{scrivpath}", ".scriv") + ".scrivx"
  scrivxpath = File.join(scrivpath,scrivx)
  # handle cases where the package has been renamed after creation
  unless File.exists?(scrivxpath)
    scrivxpath = %x{ls -1 #{scrivpath}/*.scrivx|head -n 1}.strip
  end
  return scrivxpath
end

def get_metadata(scrivxpath)
  doc = REXML::Document.new File.new(scrivxpath)
  props = REXML::XPath.first( doc.root, "ProjectProperties" )
  properties = {}
  if props
    props.each_element_with_text {|e|
      properties[e.name.downcase] = e.text
    }
  end
  properties
end

# Take the path to the scrivener file and open the internal XML file.
def get_rtf_file_array(scrivpath,scrivxpath)
  files = []
  doc = REXML::Document.new File.new(scrivxpath)
  doc.elements.each('ScrivenerProject/Binder/BinderItem') do |ele|
    if ele.attributes['Type'] == "DraftFolder"
      get_children(ele,scrivpath,files,1)
    end
  end

  return files
end

def scrivParse(input,properties)
  if RUBY_VERSION.to_f > 1.9
    input = input.force_encoding('utf-8')
  end

  input.gsub!(/<\$(\w+)>/) {|match|
    key = Regexp.last_match(1).downcase
    if properties.has_key?(key)
      properties[key]
    else
      Regexp.last_match(0)
    end
  }
  input.gsub!(/^\s*\{\\Scrv_ps=(.*?)\\end_Scrv_ps\}\s*/m,%Q{\n<pre markdown="0"><code>\\1</code></pre>\n})
  input.gsub!(/\{\\Scrv_ps=(.*?)\\end_Scrv_ps\}/m,%Q{<code>\\1</code>})
  input.gsub!(/\{\\Scrv_annot(?:.*?)\\text=(.*?)\\end_Scrv_annot\}/m,%Q{<span class="mkscriv_annotation">\\1</span>})
  input.gsub!(/XXXBREAKXXX/, "\n\n<!--BREAK-->\n\n")
  input.gsub!(/XXXLINEBREAKXXX/, "  \n")
  footnotes = []
  fncount = 0
  input.gsub!(/\{\\Scrv_fn=(.*?)\\end_Scrv_fn\}/m) {|match|
    # link = "[^fn" + rand(99999).to_s + "]"
    fncount += 1
    link = "[^fn" + fncount.to_s + "]"
    footnotes.push(link + ": " + $1)
    link
  }

  return input + "\n\n" + footnotes.join("\n\n") + "\n\n"
end

path = ARGV[0].sub(/\/$/,'')
scrivxpath = expand_scriv_path(path)

useheaders = ARGV[1] && ARGV[1] == "true" ? true : false
cache_folder = ARGV[2] ? File.expand_path(ARGV[2]) : File.dirname(File.expand_path(path))
files = get_rtf_file_array(path,scrivxpath)
properties = get_metadata(scrivxpath)
notetext = ""
files.each_with_index{ |f, idx|
  if f['path']
    contents = IO.read(f['path']).force_encoding('utf-8')
    contents = contents.extract_images(cache_folder)
    contents = contents.convert_linked_images
    contents.gsub!(/\\page /,%Q{XXXBREAKXXX})
    contents.gsub!(/\\uc0\\u8232 /,%Q{XXXLINEBREAKXXX})
    text = %x{/bin/echo #{Shellwords.escape(contents)} | textutil -convert html -format rtf -stdout -stdin}
    note = text.markdownify
  else
    note = ''
  end
  # note = f['path'] ? %x{/usr/bin/textutil -convert txt -stdout "#{f['path']}"} : ''
  # note = note.force_encoding('utf-8') if note.respond_to?('force_encoding')

  if useheaders && f['title'] != ''
    unless idx == 0 && f['title'].downcase == 'metadata'
      leader = "#"
      f['depth'].times { leader += "#" } unless f['depth'].nil?
      notetext += "#{leader} #{f['title']}\n\n"
    end
  end
  notetext += note + "\n\n"
}
# write the result to the preview file
# watch_note = File.new("#{sw_note}",'w')
# watch_note.puts notetext
# watch_note.close
# print scrivParse(notetext)
print scrivParse(notetext,properties).strip
